package com.muyu.system.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.constant.CacheConstants;
import com.muyu.common.core.utils.JwtUtils;
import com.muyu.common.redis.service.RedisService;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import com.muyu.common.system.domain.SysUser;
import com.muyu.system.domain.Number;
import com.muyu.system.domain.SysNotice;
import com.muyu.system.domain.req.NoticeListReq;
import com.muyu.system.domain.resp.NoticeRes;
import com.muyu.system.domain.vo.NoticeVo;
import com.muyu.system.mapper.SysNoticeMapper;
import com.muyu.system.service.SysNoticeService;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 公告 服务层实现
 *
 * @author muyu
 */
@Service
public class SysNoticeServiceImpl extends ServiceImpl<SysNoticeMapper, SysNotice> implements SysNoticeService {
    @Autowired
    private SysNoticeMapper noticeMapper;


    @Autowired
    private HttpServletRequest request;



    @Autowired
    private RedisService redisService;


    /**
     * 查询公告信息
     *
     * @param noticeId 公告ID
     * @return 公告信息
     */
    @Override
    public SysNotice selectNoticeById(Long noticeId) {
        return noticeMapper.selectNoticeById(noticeId);
    }

    /**
     * 查询公告列表
     *
     * @param notice 公告信息
     * @return 公告集合
     */
    @Override
    public List<SysNotice> selectNoticeList(SysNotice notice) {
        return noticeMapper.selectNoticeList(notice);
    }

    /**
     * 新增公告
     *
     * @param notice 公告信息
     * @return 结果
     */
    @Transactional
    @Override
    public int insertNotice(NoticeVo notice) {

        int i = noticeMapper.insertNotice(notice);
        if (CollectionUtils.isNotEmpty(notice.getUserIds())) {
            noticeMapper.insertInformUserId(notice.getNoticeId(), notice.getUserIds());
        }
        return i;
    }

    /**
     * 修改公告
     *
     * @param notice 公告信息
     * @return 结果
     */
    @Override
    public int updateNotice(SysNotice notice) {
        return noticeMapper.updateNotice(notice);
    }

    /**
     * 删除公告对象
     *
     * @param noticeId 公告ID
     * @return 结果
     */
    @Override
    public int deleteNoticeById(Long noticeId) {
        return noticeMapper.deleteNoticeById(noticeId);
    }

    /**
     * 批量删除公告信息
     *
     * @param noticeIds 需要删除的公告ID
     * @return 结果
     */
    @Override
    public int deleteNoticeByIds(Long[] noticeIds) {
        return noticeMapper.deleteNoticeByIds(noticeIds);
    }



    @Override
    public List<NoticeRes> findList(NoticeListReq req) {
        String token = SecurityUtils.getToken(request);
        String userKey = JwtUtils.getUserKey(token);
        LoginUser loginUser = redisService.getCacheObject(CacheConstants.LOGIN_TOKEN_KEY + userKey);
        SysUser sysUser = loginUser.getSysUser();
        Long deptId = sysUser.getDeptId();
        Long userId = sysUser.getUserId();
        req.setDeptId(deptId);
        req.setUserId(userId);
        return noticeMapper.findList(req);
    }


    @Override
    public NoticeRes look(Integer id, Integer state) {
        if (state==1){
            noticeMapper.updNoticeState(id);
        }
        return noticeMapper.look(id);
    }

    @Override
    public Boolean addNoticesUserIdAndNoticeId(Long userId, List<SysNotice> sysNotices) {
        return noticeMapper.addNoticesUserIdAndNoticeId(userId,sysNotices)>0;
    }

    @Override
    public Number NoticeCount(Integer id) {
        int sum = noticeMapper.NoticeCount(id);
        int pageview = noticeMapper.NoticePageview(id);
        int unread=sum-pageview;
        Number number = new Number();
        number.setSum(sum);
        number.setPageview(pageview);
        number.setUnread(unread);
        return number;

    }

    @Override
    public int NoticePageview(Integer id) {
        return noticeMapper.NoticePageview(id) ;
    }


}
